// ../shared/src/ts.utils.ts
function never(neverCheck, value) {
  throw new Error(
    `Unexpected value, received: ${JSON.stringify(value ?? neverCheck, null, 2)}`
  );
}

// src/openai/openai.runner.ts
import zodToJsonSchema from "zod-to-json-schema";
var OpenAIUtils = class _OpenAIUtils {
  static models = {
    GPT_3_5_TURBO: "gpt-3.5-turbo",
    GPT_4: "gpt-4",
    GPT_4_TURBO: "gpt-4-turbo",
    GPT_4O: "gpt-4o",
    GPT_4O_MINI: "gpt-4o-mini"
  };
  static mapCompletionQuery(query) {
    const systemMessages = _OpenAIUtils.systemMessages(query.system);
    const messages = query.messages.map(_OpenAIUtils.mapMessage);
    return {
      // Todo: repair
      model: "query.model",
      max_tokens: query.maxTokens,
      temperature: query.temperature,
      messages: [...systemMessages, ...messages],
      tools: _OpenAIUtils.tools(query.toolsConfig),
      tool_choice: _OpenAIUtils.toolChoice(query.toolsConfig)
    };
  }
  static systemMessages(instructions) {
    if (!instructions) return [];
    return [
      {
        role: "system",
        content: instructions
      }
    ];
  }
  static tools(toolsConfig) {
    if (toolsConfig === void 0) return void 0;
    if (toolsConfig.type === "multi")
      return toolsConfig.tools.map(_OpenAIUtils.mapTool);
    if (toolsConfig.type === "single")
      return [_OpenAIUtils.mapTool(toolsConfig.tool)];
    never(toolsConfig);
  }
  static toolChoice(toolsConfig) {
    if (toolsConfig === void 0) return "none";
    if (toolsConfig.type === "single")
      return {
        type: "function",
        function: {
          name: toolsConfig.tool.name
        }
      };
    if (toolsConfig.requireToolUse) return "required";
    return "none";
  }
  static mapTool(functionCall) {
    const schema = zodToJsonSchema(functionCall.schema);
    delete schema.$schema;
    return {
      type: "function",
      function: {
        name: functionCall.name,
        description: functionCall.schema?.description,
        parameters: schema
      }
    };
  }
  static mapMessage(message) {
    if (message.role === "assistant")
      return message;
    if (message.role === "user")
      return {
        role: "user",
        content: message.content.map(_OpenAIUtils.mapMessageContent)
      };
    throw new Error("Unknown message role");
  }
  static mapMessageContent(messagePart) {
    if (messagePart.type === "text")
      return messagePart;
    if (messagePart.type === "image")
      return {
        type: "image_url",
        image_url: {
          url: messagePart.image.data
        }
      };
    throw new Error("Unknown message content type");
  }
};
export {
  OpenAIUtils
};
//# sourceMappingURL=openai.index.js.map